import logging

from django.contrib.auth.models import User
from django.db import IntegrityError
from django.db.models.signals import post_save
from django.dispatch import receiver
from tendenci.apps.user_groups.models import Group, GroupMembership

logger = logging.getLogger(__name__)

SUFFIX = '.edu'


def _get_or_create_email_group(suffix):
    """Create a system generated group for this suffix."""
    try:
        group = Group.objects.get(name=SUFFIX)
    except Group.DoesNotExist:
        # Extremely small risk of race condition here. We can't use
        # get_or_create though, as the get won't match if anyone manually
        # modifies various attributes of the group (which has happened).
        group = Group.objects.create(
            name=SUFFIX, type=Group.TYPE_SYSTEM_GENERATED,
            allow_self_add=False, allow_self_remove=False,
            notes='Auto-generated by the group_on_email app.')
    return group


def reset_email_group(suffix=SUFFIX):
    """Clear an regenerate the group members for this suffix."""
    group = _get_or_create_email_group(suffix);
    logger.info(f'Clearing group: {group}.')
    GroupMembership.objects.filter(group=group).delete()
    users = User.objects.filter(email__iendswith=suffix)
    for user in users:
        GroupMembership.add_to_group(member=user, group=group)
        logger.info(f'Added {user} to group {group}.')


@receiver(post_save, sender=User)
def user_saved(sender, **kwargs):
    """Update email group membership on User save."""
    user = kwargs['instance']
    logger.info(f'User saved: {user}')
    group = _get_or_create_email_group(SUFFIX)
    if user.email.endswith(SUFFIX):
        try:
            GroupMembership.add_to_group(member=user, group=group)
        except IntegrityError:
            pass
        else:
            logger.info(f'Added {user} to group {group}.')
    else:
        GroupMembership.objects.filter(member=user, group=group).delete()
        logger.info(f'Removed {user} from group {group}.')
